DKI.Assessment = $.Class({
	prototype: {
		init: function(module, prepost, test) {
			this.module = module;
			this.unanswered = test.questions.concat();			
			this.answered = [];
			this.prepost = prepost;
			this.currentQuestionIndex = 0;
			this.currentQuestion = this.unanswered[this.currentQuestionIndex];
			this.nextQuestion = this.getNextQuestion();
			this.previousQuestion = this.getPreviousQuestion();
			this.totalQuestions = this.unanswered.length;
			this.currentQuestionCount = 1;
			this.totalWeight = 0;
			for (var i = 0; i < test.questions.length; i++) {
				this.totalWeight += parseInt(test.questions[i].weight, 10);
			} 
		},
		getScore: function () {
			var score = 0;	
			if (this.answered.length > 0) {
				for (var i = 0; i < this.answered.length; i ++) {
					var weight = parseInt(this.answered[i].weight, 10);
					score += parseFloat(this.answered[i].score, 10) * weight;
				}
				if (score == 0 && this.totalWeight == 0) {
					score = 0;
				}
				else {
					score = score / this.totalWeight;
				}
			}			
			return Math.round(score);
		},
		getPoints: function(){
			var points = parseFloat(0, "10");
			if (this.answered.length > 0) {
				for (var i = 0; i < this.answered.length; i ++) {
					points += parseFloat(this.answered[i].points, "10");
				}				
			}			
			return points.toFixed(1);
		},
		getWeight: function () {			
			return this.totalWeight;
		},
		getNextQuestion: function () {
			var nextIndex = this.currentQuestionIndex + 1;
			if (this.unanswered.length === 1) {
				//We are answering the last question.  Set next question to null, so player knows
				//to load endscreen next
				return null;
			}
			else if (nextIndex >= this.unanswered.length) {
				nextIndex = 0;
			}

			return this.unanswered[nextIndex];
		},
		getPreviousQuestion: function () {
			var previousIndex = this.currentQuestionIndex - 1;
			if (this.unanswered.length === 1) {
				//We are answering the last question.  Return null
				return  null;
			}
			else if (previousIndex < 0) {
				previousIndex = this.unanswered.length - 1;
			}

			return this.unanswered[previousIndex];
		},
		goNextQuestion: function () {
			if (this.currentQuestion !== null) {
				this.currentQuestionIndex ++;
			}
			if (this.currentQuestionIndex >= this.unanswered.length) {
				this.currentQuestionIndex = 0;
			}

			if (this.unanswered.length > 0) {
				this.currentQuestion = this.unanswered[this.currentQuestionIndex];
				this.nextQuestion = this.getNextQuestion();
				this.previousQuestion = this.getPreviousQuestion();
			}
			else {
				this.currentQuestion = null;
				this.nextQuestion = null;
				this.previousQuestion = null;
			}
		},
		goPreviousQuestion: function () {
			if (this.currentQuestionIndex > 0) {
				this.currentQuestionIndex --;
			}
			else {
				this.currentQuestionIndex = this.unanswered.length - 1;
			}

			this.currentQuestion = this.unanswered[this.currentQuestionIndex];
			this.nextQuestion = this.getNextQuestion();
			this.previousQuestion = this.getPreviousQuestion();
		},
		scoreQuestion: function (question, questionObject, score) {
			var questionPage;
			for (var i = 0; i < this.unanswered.length; i ++) {
				if (this.unanswered[i].id === question.id) {
					questionPage = this.unanswered[i].page;
					this.unanswered.splice(i, 1);
					this.answered[this.answered.length] = {
						id: question.id, 
						pageid: question.pageId, 
						choices: questionObject.options, 
						score: score, 
						body: questionObject.body,
						page: questionPage,
						options: question.options,
						weight: question.weight,
						points: parseFloat(score * question.weight / 100).toFixed(1)
					};
					this.currentQuestionIndex = i;
				}
			}
			if (this.unanswered.length > 0) {
				this.currentQuestionCount ++;
			}
			this.currentQuestion = null;
		}

	}
});
