/**
 * @module player 
 * @description This module contains the classes necessary to implement the LCMS course player, used to preview courses online.
 */
var DKI;
if (!DKI) {
	DKI = {};
}
/**
 * Class used to implement the assessment preview player
 * @class Assessment
 * @namespace DKI.lcms
 */
DKI.Assessment = function (loid, prepost, authToken) {
	this.test;
	this.currentQuestion = 0;
	this.currentQuestionDisplay = 0;
	this.loid = loid;
	this.prepost = prepost;
	this.submitted = false;
	this.feedbackClosed = false;
	this.authToken = authToken;
	this.initEvents();
};

DKI.Assessment.prototype.initEvents = function () {
	this.loadComplete = "loadComplete";
	this.loadQuestion = "loadQuestion";
	this.loadEndScreen = "loadEndScreen";
};

DKI.Assessment.prototype.load = function () {
	var ajaxArgs = {
		headers: {"X-dki-authtoken": this.authToken},
		type: "GET",
		url: "../api/student.cfm/module/" + this.loid + "/starttest",
		context: this,
		success: function (data, textStatus, jqXHR) {
			this.test = data,
			this.totalQuestions = this.test.questions.length;
			this.currentQuestionDisplay = 1;
			$(this).trigger(this.loadComplete, ["../ajax/course.cfm/renderPage/" + this.test.questions[this.currentQuestion].pageid + ".html"]);
		},
		error: function (jqXHR, textStatus, errorThrown) {
			alert("Failed to load assessment " + textStatus);
		}
	};
	$.ajax(ajaxArgs);
};

//Instance methods

/**
 * Moves to the previous question in the list
 */
DKI.Assessment.prototype.back = function () {
	this.currentQuestion--;
	if (this.currentQuestion < 0) {
		this.currentQuestion = this.test.questions.length - 1;
	}
	this.loadCurentQuestion();
};

/**
 * Moves to the next question in the list
 */
DKI.Assessment.prototype.forward = function () {
	this.currentQuestion++;
	if (this.currentQuestion >= this.test.questions.length) {
		this.currentQuestion = 0;
	}
	this.loadCurrentQuestion();
};

/**
 * Loads the current question in the content frame
 */
DKI.Assessment.prototype.loadCurrentQuestion = function () {
	this.feedbackClosed = false;
	this.submitted = false;
	if (this.test.questions.length) {
		this.currentQuestionDisplay++;
		$(this).trigger(this.loadQuestion, ["../ajax/course.cfm/renderPage/" + this.test.questions[this.currentQuestion].pageid + ".html"]);
	}
	else {
		$(this).trigger(this.loadEndScreen, ["endtestinform.cfm?thiscurrentlo=" + this.loid + "&prepost=" + this.prepost + "&testid=" + this.test.id]);
	}
};

/**
 * Submits question response back to the server
 * @param {String[]} choices The user's responses
 */
DKI.Assessment.prototype.submit = function (choices) {
	var ajaxArgs = {
		headers: {"X-dki-authtoken": this.authToken},
		type: "POST",
		context: this,
		dataType: "json",
		success: function (data, textStatus, jqXHR) {
			this.submitted = true;
			this.test.questions.splice(this.currentQuestion, 1);
			if(this.currentQuestion >= this.test.questions.length) {
				this.currentQuestion = 0;
			}
			if (this.feedbackClosed) {
				this.loadCurrentQuestion();
			}
		},
		error: function (jqXHR, textStatus, errorThrown) {
			alert("Submit failed " + + jqXHR.status + " " + textStatus);
		},
		data: "",
		url: "../api/student.cfm/test/" + this.test.id + "/question/" + this.test.questions[this.currentQuestion].id
	}
	for (var i = 0; i < choices.length; i++) {
		ajaxArgs.data += "&choice_" + (i + 1) + "=" + choices[i];
	}
	$.ajax(ajaxArgs);
};

/**
 * Opens the design notes interface
 */
DKI.Assessment.prototype.notes = function () {
};

/**
 * Called by the test question when the feedback box is closed.
 */
DKI.Assessment.prototype.onFeedbackClose = function () {
	this.feedbackClosed = true;
	if (this.submitted) {
		this.loadCurrentQuestion();
	}
};
