/**
 * Extends the datastore object to contained a store construct flatted to object keys
 * with attribbutes for consumption by tree and other ui elements
 * @extends	DKI
 * @class	courseStore
 */

DKI.courseStore = function(){

	var courseNode = null 	// needed for initial calls

	/**
	 * Nodes types contained within the tree
	 * @extends DKI.courseSTore
	 */
	var NodeType = {
		course	: 'course',
		module	: 'module',
		object	: 'object',
		page	: 'page',
		question: 'question'
	}; //-> end NodeType

	/**
	 * Returns the base node configuration 
	 * @private baseClass 
	 */
	function baseClass(){
		return {
			complete 	: false,
			completeCnt : 0,
			nodeType	: NodeType.course,
			id 			: null,
			key			: null,
			parentId	: null,
			pageNote	: null,
			transcript	: null,
			childKeys	: [],
			name		: ''
		}
	}

	/**
	 * Holder for loaded store
	 * @private store
	 */
	var store = {};
	
	/**
	 * Loads the store with the passed data
	 * @extends DKI.courseStore
	 * @method	Load
	 * @param	data required dataStorage.courseStructure
	 * @return	truee
	 */
	var Load = function(data){
	
		var testOnly = data.testOnly;
		var storeKey = NodeType.course + data.id
		store[storeKey]=DKI.applyIf(
			DKI.apply({
					nodeType	: NodeType.course,
					key			: storeKey,
					parentId	: storeKey
				},data),
			new baseClass());
		delete store[storeKey].modules;
		
		courseNode=storeKey
		
		for (var i = 0; i < data.modules.length; i++){
			
			var module = data.modules[i];
			var moduleKey = NodeType.module + module.id	
						
			store[moduleKey]=DKI.applyIf(
				DKI.apply({
						nodeType	: NodeType.module,
						key			: moduleKey,
						parentId	: storeKey,
						questionKeys:[]
					},module),
				new baseClass());
								
			delete store[moduleKey].objects;
			
			store[storeKey].childKeys.push(moduleKey);	
			
			for (var j = 0; j < module.objects.length; j++) {
				
				var object = module.objects[j];
				var objectKey = NodeType.object + object.id	
							
				store[objectKey]=DKI.applyIf(
					DKI.apply({
							nodeType	: NodeType.object,
							key			: objectKey,
							parentId	: moduleKey
						},object),
					new baseClass());
									
				delete store[objectKey].pages;
				
				store[moduleKey].childKeys.push(objectKey);	
				
				// add the pages in
				for (var k = 0; k < object.pages.length; k++) {
					
					var page = object.pages[k];
					var pageKey = NodeType.page + page.id	
								
					store[pageKey]=DKI.applyIf(
						DKI.apply({
								nodeType	: NodeType.page,
								key			: pageKey,
								parentId	: objectKey,
								moduleKey	: moduleKey
							},page),
						new baseClass());
					
					store[objectKey].childKeys.push(pageKey);							
				} // end page loop
				
				// add the questions in only if there are pages to display, or the course is testOnly
				if (object.pages.length > 0 || testOnly == "1") {
					for (var k = 0; k < object.questions.length; k++) {
						
						var question = object.questions[k];
						var questionKey = NodeType.question + question.id	
									
						store[questionKey]=DKI.applyIf(
							DKI.apply({
									nodeType	: NodeType.question,
									key			: questionKey,
									parentId	: objectKey,
									pageId		: question.pageid,
									transcript	: ''
								},question),
							new baseClass());
						
						// since visually questions belong to a module we 
						// push them into the module keys
						store[moduleKey].questionKeys.push(questionKey);
						store[objectKey].childKeys.push(questionKey);						
					} // end page loop
				}
								
			} // end object loop
		} // end module loop
		
		return true;
	}; // end load
			
	/**
	 * Returns the course node element
	 * @extends	DKI.courseStore.course
	 * @method	get
	 * @returns	course base record
	 */		
	var getCourse = function(){
		return store[courseNode];
	};
	
	/**
	 * Returns the course node element
	 * @extends	DKI.courseStore.course
	 * @method	set
	 * @param	o As the course object to set as the course object
	 * @returns	null
	 */
	var setCourse = function(o){
		store[courseNode]=o;
		return null;
	};;

	/**
	 * Returns an ordered list of all module objects in the course
	 * @extends	DKI.courseStore.module
	 * @method	all
	 * @returns	array of records
	 */
	var getAllModules = function(){
		var result=[];
		for(m in store[courseNode].childKeys){
			result.push(store[store[courseNode].childKeys[m]]);
		};
		return result;
	};
	
	/**
	 * Returns the course node element
	 * @extends	DKI.courseStore.module
	 * @method	get
	 * @param	key required string key of module record to fetch
	 * @returns	module record or null if not found
	 */	
	var getModuleAtKey = function(key){
		return store[key].nodeType==NodeType.module ? store[key]: null;
	};

	/**
	 * Sets the module record to the passed module record
	 * @extends	DKI.courseStore.module
	 * @method	set
	 * @param	o as the module object to set as the module object
	 * @returns	null
	 */	
	var setModule = function(o){
		store[o.key]=o;
		return null;
	};
	
	/**
	 * Returns an ordered list of all course objects
	 * @extends	DKI.courseStore.object
	 * @method	all
	 * @returns	array of objects
	 */	
	var getAllObjects = function(){
		var result=[];
		for(m in store[courseNode].childKeys){
			for(o in store[store[courseNode].childKeys[m]].childKeys){
				result.push(store[store[store[courseNode].childKeys[m]].childKeys[o]])
			}
		};
		return result
	};

	/**
	 * Returns the object record at key
	 * @extends	DKI.courseStore.object
	 * @method	get
	 * @param	key required module key
	 * @returns	the requested object record or null if not found
	 */	
	var getObjectAtKey = function(key){
		return store[key].nodeType==NodeType.object ? store[key]: null;
	};
	
	/**
	 * Sets the object record to the passed object record
	 * @extends	DKI.courseStore.object
	 * @method	set
	 * @param	o as the object record to set as the object to
	 * @returns	null
	 */		
	var setObject = function(o){
		store[o.key]=o;
		return null
	};	
	
	/**
	 * Returns an ordered list of all course pages
	 * @extends	DKI.courseStore.page
	 * @method	all
	 * @returns	array of pages
	 */		
	var getAllPages = function(){
		var result=[];
		for(m in store[courseNode].childKeys){
			var mKey=store[store[courseNode].childKeys[m]];
			for(o in mKey.childKeys){
				var oKey=store[mKey.childKeys[o]]
				for(p in oKey.childKeys){
					var pKey = store[oKey.childKeys[p]]
					result.push(pKey)
				}
			}
		};
		return result
	};
	
	/**
	 * Returns the page record at key
	 * @extends	DKI.courseStore.page
	 * @method	get
	 * @param	key required page key
	 * @returns	the requested page record or null if not found
	 */		
	var getPageAtKey = function(key){
		return store[key].nodeType==NodeType.page ? store[key]: null;
	};
	
	/**
	 * Sets the page object based on the passed page object
	 * @extends	DKI.courseStore.page
	 * @method	set
	 * @param	o as the page record to set the page object to
	 * @returns	null
	 */		
	var setPage = function(o){
		store[o.key]=o;
		return null;
	};		
	
	/**
	 * This function is an iterative function, in that it calls itself so care should be take.
	 * Takes a storeKey and increments its parent counter and sets the parent complete indicator.
	 * @extends DKI.courseStore
	 * @method	SetParentCount
	 * @param	Key
	 * @return	The status of the last node found to be complete
	 */	
	var SetParentCount = function(key){
		
		if(store[key]==undefined){return false};
		
		var parentNode = store[store[key].parentId];
		parentNode.completeCnt++;
		parentNode.complete = parentNode.completeCnt >= (parentNode.nodeType==NodeType.object ? parentNode.childKeys.length - parentNode.questions.length : parentNode.childKeys.length);
		
		if(parentNode.complete && parentNode.key!==parentNode.parentId){
			SetParentCount(parentNode.key)
		}
		return parentNode.complete	
		
	}; //-> end setParentCount
	
	/**
	 * Gets the node from the store based on the full key
	 * @extends DKI.courseStore
	 * @method	get
	 * @param	key	Required valid String store key
	 * @return	requested record or null if not found 
	 */
	var get = function(key){
		return store[key]==undefined ? null : store[key]
	}; //-> end get	

	var set = function(o){
		store[o.key]=o;
		return store[o.key];		
	}; //-> end get	
		
	return	{
		load			: Load,
		setParentCount	: SetParentCount,
		NodeType		: NodeType,
		get 			: get,
		set 			: set,
		course			: {
			get 	: getCourse,
			set		: setCourse
		},
		module			: {
			all		: getAllModules,
			get 	: getModuleAtKey,
			set		: setModule
		},
		object			: {
			all		: getAllObjects,
			get 	: getObjectAtKey,
			set 	: setObject  
		},
		page 			: {
			all 	: getAllPages,
			get 	: getPageAtKey,
			set 	: setPage
		},
		store:store
	};		
					
}(); //-> end courseStore
