var DKI;
if (!DKI){DKI={};}

DKI.EndCourse = function () {
	var certificateLink;
	var certNotAvailable;
	var dataStorage;
	var behaviour;
	var currentModule;
	var courseScoringDetails;
	var courseCompletion;
	var player;
	var remainingAttempts;
	var modsWithTest;
	var testingDisplay;
	var courseTestingDetails;
	var courseScoreValue;
	var coursePassingStatus;
	var testingList;
	var strings;
	var pageContainer;
	var coursePassingScoreLabel;

	var init = function (playerObj, pageEl) {
		dataStorage = playerObj.dataStorage;
		course = playerObj.course;
		behaviour = playerObj.behaviour;
		currentModule = dataStorage.currentModule;
		player = playerObj;
		strings = playerObj.strings;
		pageContainer = pageEl;
		$(document).trigger(DKI.EndModule.events.ready);
	};

	var start = function () {
		certificateLink = $("#endCourse_certificateLink", pageContainer);	
		certNotAvailable = $("#endCourse_certificateNotAvailable", pageContainer);
		testingDisplay = $("#endCourse_testing", pageContainer);
		testingList = $("ul", testingDisplay[0]);
		coursePassingScoreLabel = $("#endCourse_status .passing_score_label", pageContainer);
		courseScoringDetails = dataStorage.getCourseScoringDetails();
		courseCompletion = dataStorage.getCourseCompletion();
		remainingAttempts = dataStorage.remainingPostAttempts(currentModule);
		if (behaviour.displayPass && courseScoringDetails.weight > 0) {
			$(".passing_score_container", pageContainer).addClass("visible");
		}
		if (behaviour.enablePostTest) {
			modsWithTest = dataStorage.modulesWithTesting();
			if (modsWithTest.length > 0) {
				for (var i = 0; i < modsWithTest.length; i ++) {
					testingList.append(createModuleTestItem(modsWithTest[i]));
					if (i === modsWithTest.length -1) {
						$("li:last-child", testingList).addClass('dki-lineitem-last');
					}
				}
				
				testingDisplay.delegate("li a", "click", function (e) {
					player.launchTest($(this).data("moduleid"), "post");	
				});

			}
		}
		else {
			testingDisplay.hide();
		}
		dataStorage.checkCourseComplete();

		if (!isNaN(parseInt(courseCompletion))) {
			$("#endCourse_learningStatus", pageContainer).text(courseCompletion + "%");
		}
		else {
			$("#endCourse_learningLine", pageContainer).hide();
		}
		if (!isNaN(parseInt(courseScoringDetails.score, 10)) && dataStorage.hasTesting()) {
			courseTestingDetails = $("#endCourse_testingStatus", pageContainer);
			courseScoreValue = $("#course_score", courseTestingDetails);
			coursePassingStatus = $("#course_passing_status", pageContainer);
			courseScoreValue.text(Math.round(courseScoringDetails.score) + "%" + " (" + courseScoringDetails.points + 
				"/" + courseScoringDetails.weight + " " + strings.txtPoints + ")");
			if (behaviour.displayPass && courseScoringDetails.weight > 0) {
				//We just want to display pass/fail status passed on current score vs. passing score, without
				//worrying about actual completion criteria
				courseTestingDetails.removeClass("dki-themeTextColor");
				if (dataStorage.lessonStatus === "passed") {
					courseTestingDetails.addClass("pass");	
					var statusText = strings.passText;
					var statusClass = "pass";
				}
				else if (!dataStorage.allTestsAttempted()) {
					courseTestingDetails.addClass("incomplete");
					var statusText = strings.incompleteText;
					var statusClass = "dki-themeTextColor";
				}
				else {
					courseTestingDetails.addClass("fail");
					var statusText = strings.failText;
					var statusClass = "fail";
				}
				coursePassingStatus.text(" - " + statusText + " - ").addClass(statusClass);
			}
			else {
				coursePassingStatus.css("display", "none");
			}
		}
		else {
			$("#endCourse_testingStatus", pageContainer).text("N/A");
			$("#endCourse_testing", pageContainer).hide();
			$("#testingLine", pageContainer).hide();
			$("#endCourse_learningLine", pageContainer).addClass("dki-lineitem-last");
		}

		if (behaviour.completionCertificate) {
			$(".endScreen", pageContainer).addClass("showCertificate");
		}

		if (dataStorage.lessonStatus === "complete" || dataStorage.lessonStatus === "passed") {
			certificateLink.removeClass("disabled");
			certNotAvailable.css("display", "none");
			certificateLink.click(function (e) {
				player.loadCertificate();
			});
		}
		
		$("#endCourse_testLink", pageContainer).click(function (e) {
		this.contentFrame.attr("src", this.settings.endCourseURL);
			player.launchTest(currentModule.id, "post");
		});
	};

	/**
	 * Creates & returns HTML element for displaying a module's test details
	 */
	var createModuleTestItem = function (module) {
		var modName = module.name;	
		var modScoringDetails = dataStorage.getModuleScoringDetails(module);
		var modAttempts;
		var modStatusText;
		var modID;
		var listItem;
		var passingDetailsItem;
		var passingMarkItem;
		var passingMarkValueItem;
		var passingStatusValueItem;
		var modPassingScore;
		var createTitleItem = function () {
			//Create/populate item holding the module title
			var titleItem = document.createElement("span");
			titleItem.className = "dki-modTitle";
			titleItem.innerHTML = modName;

			return titleItem;
		};
		var createScoreItem = function () {
			//Create/populate item holding the module score
			var scoreItem = document.createElement("span");
			var scoreValueItem = document.createElement("span");
			var modScoreDisplay = modScoringDetails.score === null ? " Not Attempted" : " " + Math.round(modScoringDetails.score) + 
				"%" + " (" + modScoringDetails.points + "/" + modScoringDetails.weight + " " + strings.txtPoints + ")";
			scoreItem.className = "dki-modScore";
			scoreItem.innerHTML = strings.txtScore;
			if (behaviour.displayPass && behaviour.passModules) {
				//We are requiring modules be passed, and are displaying passing details
				if (module.passMark) {
					modPassingScore = module.passMark;
				}
				else {
					modPassingScore = dataStorage.courseStructure.passMark;
				}
				if (modScoringDetails.score >= modPassingScore) {
					scoreValueItem.className = "pass";
				}
				else {
					scoreValueItem.className = "fail";
				}
			}
			else {
				scoreValueItem.className = "dki-themeTextColor";
			}
			scoreValueItem.innerHTML = modScoreDisplay;
			scoreItem.appendChild(scoreValueItem);

			return scoreItem;
		};

		var createAttemptItem = function () {
			var modAttemptDisplay;
			var attemptsItem = document.createElement("span");
			var attemptsValueItem = document.createElement("span");
			modAttempts = dataStorage.remainingPostAttempts(module);
			if(modScoringDetails.weight > 0 || modScoringDetails.weight == null){
				modAttemptDisplay = " " + modAttempts;
			}
			else{
				modAttemptDisplay = " N/A";
			}
			attemptsItem.className = "dki-modAttempts";
			attemptsItem.innerHTML = strings.txtRemainingAttempts;
			attemptsValueItem.className = "dki-themeTextColor";
			attemptsValueItem.innerHTML = modAttemptDisplay;
			attemptsItem.appendChild(attemptsValueItem);

			return attemptsItem;
		};

		var createActionsItem = function () {
			var actionsItem = document.createElement("span");
			var actionButton = document.createElement("a");
			actionsItem.className = "actions";
			actionButton.className = "dki-modBtn";
			actionButton.setAttribute("data-moduleid", module.id);
			actionButton.tabindex="0";
			actionButton.role="button";
			actionButton.innerHTML = strings.txtTakeTest;
			actionsItem.appendChild(actionButton);

			return actionsItem;
		};

		var createTestDetailsItem = function () {
			var passingScore = module.passMark ? module.passMark : course.passMark;
			var detailsItem = document.createElement("div");
			var passingScoreLabel = document.createElement("span");
			var passingScoreValue = document.createElement("span");
			var passingStatus = document.createElement("span");
			detailsItem.className = "dki-modTestDetails";
			passingScoreLabel.innerHTML = coursePassingScoreLabel.text();
			passingScoreValue.innerHTML = " " + passingScore + "%";
			passingScoreValue.className = "dki-themeTextColor";
			if (modScoringDetails.score === null) {
				passingStatus.className = "dki-themeTextColor";
				passingStatus.innerHTML = " - " + strings.incompleteText + " - ";
			}
			else if (modScoringDetails.score >= passingScore) {
				passingStatus.className = "pass";
				passingStatus.innerHTML = " - " + strings.passText + " - ";
			}
			else {
				passingStatus.className = "fail";
				passingStatus.innerHTML = " - " + strings.failText + " - ";
			}
			passingStatus.className += " modulePassingStatus";
			detailsItem.appendChild(passingScoreLabel);
			detailsItem.appendChild(passingScoreValue);
			detailsItem.appendChild(passingStatus);

			return detailsItem;
		};

		modID = module.id;
		listItem = document.createElement("li");
		listItem.className = "dki-lineitem moduleitem";
		listItem.appendChild(createTitleItem());
		listItem.appendChild(createScoreItem());
		listItem.appendChild(createAttemptItem());
		if (modAttempts != 0 && (modScoringDetails.weight > 0 || modScoringDetails.weight === null)) {
			listItem.appendChild(createActionsItem());
		}
		if (behaviour.displayPass && behaviour.passModules) {
			listItem.appendChild(createTestDetailsItem());
			listItem.className += " showPassing";
		}

		return listItem;
	};

	return {init: init, start: start};
	
}();
