/**
 * Singleton representation of the feedback panel control
 * @created	August 22, 2011
 * @author	Keith Chadwick
 * @version	1.0
 */
DKI.FeedbackPanel = function(){
	
	var _layoutComplete = false;	// indicates if the feedback content has been rendered
	var _hidden = true;				// indicates if the feedback panel is displayed
	var _container = null;			// pointer to root el
	var _bWatchForComplete = false;	// used by the _checkState function to indicate one of three vents has occured
	var _bFeedbackState = false;	// holds the view state of the feedback window when it has been hidden by _checkState
	//var _lastPosition = 0;			// holds a placeholder of the last scroll amount in the All feedback tab
	var _oActivePage ={}			// local pointer to the current active page or question object	
	
	var _store = {};				// store pointer
	var _tabStrip;
	var pageFeedbackTab;
	var allFeedbackTab;
	var feedBackInput;
	var presentationPlayer;
	/**
	 * Creates the layout structure for the feeddback interface
	 * @private	_createLayoutStrcuture
	 * @return	null
	 */
	var _createLayoutStructure = function () {
		var elMenu = document.createElement("div");
		var elCloseButton = document.createElement('a');
		var elPageFeedback = document.createElement('div');
		var elPageFeedAll = document.createElement('div');
		var elInstructions;
		var elTextArea;
		var elBtnSubmit;
		var elBtnCancel;
		var elTitle;
		var elContent;
		$(_container).empty();
		elMenu.className = "menuHeader";
		elMenu.id = "feedbackPanelMenu";
		_container.append(elMenu);
		tabStrip = new DKI.TabSet({
			elementID: elMenu.id,
			player: presentationPlayer
		});
		pageFeedbackTab = tabStrip.createTab(strings.tabLabelPageFeedBk, "pageFeedbackTab");
		allFeedbackTab = tabStrip.createTab(strings.tabLabelAllFeedBk, "allFeedbackTab");
		tabStrip.selectTab(pageFeedbackTab.tab);
		// close button
		elCloseButton.className = 'buttonClose';
		elCloseButton.innerHTML = strings.buttonLabelClose;
		elMenu.appendChild(elCloseButton);
		
		// create the wrapper for the page feedback area
		elPageFeedback.className = 'innerBlockScroll dki-feedback-page';
		elPageFeedback.id = 'dki-feedback-page';
		
		// add the instructions in
		elInstructions = elPageFeedback.appendChild(document.createElement('div'));
		elInstructions.className = 'instructions';
		elInstructions.innerHTML = strings.txtInstructionsLabel;
		
		// add the textarea in
		elTextArea = elPageFeedback.appendChild(document.createElement('textarea'));
		elTextArea.className = 'txtArea-feedback';
		feedBackInput = $(elTextArea);
		
		// add the submit button
		elBtnSubmit = elPageFeedback.appendChild(document.createElement('button'));
		elBtnSubmit.className = 'btnSubmit';
		elBtnSubmit.innerHTML = strings.buttonLabelSubmit;		

		// add the cancel button
		elBtnCancel = elPageFeedback.appendChild(document.createElement('button'));
		elBtnCancel.className = 'btnCancel';
		elBtnCancel.innerHTML = strings.buttonLabelCancel;		
		
		// append the title
		elTitle = elPageFeedback.appendChild(document.createElement('div'));
		elTitle.className = 'title';
		elTitle.innerHTML = strings.txtTitleCurrentPage;
	
		// scrollable listing of all page feedback
		elContent = elPageFeedback.appendChild(document.createElement('div'));
		elContent.className = 'content';
					
		pageFeedbackTab.content.appendChild(elPageFeedback)
		
		// create the main element for all feedback tab area
		elPageFeedAll.className = 'innerBlockScroll dki-feedback-all';
		elPageFeedAll.id = 'dki-feedback-all';
		
		// title bar
		elTitle = elPageFeedAll.appendChild(document.createElement('div'));
		elTitle.className = 'title';
		elTitle.innerHTML = strings.tabLabelAllFeedBk;		
		
		// scrollable listing of all page feedback
		elContent = elPageFeedAll.appendChild(document.createElement('div'));
		elContent.className = 'content';

		allFeedbackTab.content.appendChild(elPageFeedAll);
		_layoutComplete = true;
		
		return null;
	};
	
	/**
	 * Binds all of the necessary events for the feedback interface
	 * @private	_bindEvents
	 * @return	null
	 */
	var _bindEvents = function(){
		// close ui [x] click		
		$('#feedbackPanel .menuHeader').delegate('.buttonClose', presentationPlayer.settings.clickEvent + ' keypress', function (e) {
			e.stopPropagation();
			//Respond to click, touchstart or enter or spacebar
			if (e.type !== "keypress" || (e.which === 13 || e.which === 32)) {
				Handler.HideFeedback();
			}
		});
		
		// submit button click
		$('#feedbackPanel .dki-feedback-page').delegate('.btnSubmit', presentationPlayer.settings.clickEvent + ' keypress', function (e) {
			e.stopPropagation();
			if (e.type !== "keypress" || (e.which === 13 || e.which === 32)) {
				Handler.SubmitFeedback();
			}
		});
		
		// cancel button click
		$('#feedbackPanel .dki-feedback-page').delegate('.btnCancel', presentationPlayer.settings.clickEvent + ' keypress', function (e) {
			e.stopPropagation();
			if (e.type !== "keypress" || (e.which === 13 || e.which === 32)) {
				Handler.ClearFeedback();
			}
		});
		$(allFeedbackTab.tab).on(tabStrip.tabSelected, function (e) {
			_highlightFeedbackPage()
		});

		
		$(dataStorage).on(DKI.DataStorage.events.pageSelected, function (e, page) {
			Handler.onPageClick(page);
		})
		$(dataStorage).on(DKI.DataStorage.events.questionSelected, function (e, question) {
			Handler.onPageClick(question.page);
		})
		
		$(dataStorage).on(DKI.DataStorage.events.pageCompleted, _checkState);
		$(dataStorage).on(DKI.DataStorage.events.changeModule, _checkState)
		$(dataStorage).on(DKI.DataStorage.events.questionSelected, _checkState)
		$(dataStorage).on(DKI.DataStorage.events.assessmentCompleted, _checkState)
		$(dataStorage).on(DKI.DataStorage.events.courseEnd, _checkState)
				
		return null;
	}; //-> end _bindEvents
	
	/**
	 * Handler subclass, all methods within the Handler singleton are bound to mouse events
	 */
	var Handler = {
		
		/**
		 * Rerenders the page feedback current container
		 * @method	onOagClick
		 * @param	e event object
		 * @param	pageID integer id of teh page to move to in the feedback area
		 * @return	null;
		 */
		// no args
		onPageClick : function(page){
			_oActivePage = page;
			_renderPageFeedback();
			_highlightFeedbackPage();
			return null;
		}, //-> onPageClick

		/**
		 * Handles The submission of the text
		 */
		SubmitFeedback : function(){
			presentationPlayer.onFeedbackSubmit(
				feedBackInput.val(),
				function(data){
			
					// insert the returned data into the store
					_oActivePage.notes.push(data)
					_oActivePage=_store.set(_oActivePage);			
							
					// Update the page feed back	
					_renderPageFeedback();
					
					// if the note length is 1 then under the all feedback tab for that
					// page it will presently say no reviewer feedback so we must first remove that
					if(_oActivePage.notes.length==1){
						$('#feedbackPanel .dki-feedback-all .content .active p').remove()
					}
					
					// append the new note to all feedback tab
					/*
					_insertFeedbackNote({
						el 		: $('#feedbackPanel .dki-feedback-all .content .active')[0],
						note 	: data,
						method	: 'prepend'
					});
					*/
					
					/**
					 * This is very inefficient and a proper prepend would be better but due to the dom
					 * structure in the allfeedback tab this is the easiest way to do it for now.
					 */
					_renderAllFeedback()
					
					
					Handler.ClearFeedback();
					return true;
				}
			)
		}, //-> end SubmitFeedback

		/**
		 * Handles the clearing of the input for on the page feedback ui
		 * @private ClearFeedback
		 * @return	true
		 */
		ClearFeedback : function(){
			feedBackInput.val('');
			return true
		}, //-> end ClearFeedback
		
		/**
		 * Handles the close of the feedback window
		 * @private	HideFeedback
		 * @return	true
		 */
		 HideFeedback : function(){
			_container.addClass('hidden');
			_hidden=true;
			return true
		} //-> end _handleCloseEvent
						
	}; //-> end Handler

	/**
	 * Function is fired fia the bound datalayer events in the local _bindEvents method
	 * @private _checkState
	 * @param	oEvent	The event object;
	 * @return	true
	 */
	var _checkState = function(oEvent){
		var sEvent=oEvent.type;
		
		// this would indicate a end of module page is being displayed so we
		// must disble feedback
		if(sEvent =='changeModule' || sEvent=='courseEnd' || sEvent=='assessmentCompleted'){
			_bFeedbackState = _hidden;
			$('#feedbackTab').css('display','none');
			hide();
			_bWatchForComplete = true;
		}else if((sEvent == 'questionSelected' || sEvent=='pageCompleted') && _bWatchForComplete){
			_bWatchForComplete = false;
			$('#feedbackTab').css('display','block');
			_bFeedbackState ? hide() : show();
		};
		
		return true;
	};
	
	/**
	 * Sets the local active page pointer to either the current page or question when in assessmen
	 * @private	_setActivePage
	 * @return	null;
	 */
	var _setActivePage = function(){
		_oActivePage = _store.get(presentationPlayer.inAssessment ? 
			_store.NodeType.question + presentationPlayer.dataStorage.assessment.currentQuestion.id : 
			_store.NodeType.page + presentationPlayer.dataStorage.currentPage.id
		);
		return null;		
	};

	/**
	 * Highlights the active page in the all feedback viewport
	 * @private	_highlightFeedbackPage
	 * @param	pageId	integer representing the page id to highlight
	 * @return	null
	 */
	var _highlightFeedbackPage = function(){
		
		if(_oActivePage==undefined || _oActivePage==null){return null};
		
		// remove any current selected 
		$('#feedbackPanel .dki-feedback-all .content .active').removeClass('active')
			
		var el=$($('#feedbackPanel .dki-feedback-all .content .slctr-' + _oActivePage.id).parent())
		el.addClass('active')
		
		$('#feedbackPanel .dki-feedback-all .content').scrollTop(el.position().top)

		return null;
	}; //-> end _highlightFeedbackPage
	
	/**
	 * Called once on initialization on first render, renders ALL of the existing notes in the All Feedback page
	 * @private	_renderAllFeedback
	 * @return	null
	 */
	var _renderAllFeedback = function(){
		
		var el=$('#feedbackPanel .dki-feedback-all .content')
		
		$(el).children().each(function(i){
			$(this).remove()
		});
				
		// grab all the page data and render the subsequent notes and questions
		// NOTE: A module must contain an object with a page in order for the questions to show up
		//		 so we therefore can key of off pages 
		var pages = _store.page.all();
		for(var iPage= 0; iPage < pages.length; iPage++){
			var page = pages[iPage];
			_createPageElements(page);
		}; 		
		
		function _createPageElements(node){
			
			var notes;
			var elPage=document.createElement('div');
			elPage.className = 'page';
			if (node.nodeType !== DKI.courseStore.NodeType.page) {
				node = node.page;
			}
			notes = node.notes;
							
			var pageTitle=elPage.appendChild(document.createElement('div'));
			pageTitle.innerHTML = node.title;
			pageTitle.className = 'slctr-' + node.id
			
			if(notes.length==0){
				var line=elPage.appendChild(document.createElement('p'))
				line.innerHTML= strings.txtTitleNoFeedBack;
				$(line).css('margin-bottom','20px')
			}else{
								
				for(var iNote in notes){
					_insertFeedbackNote({
						el 		: elPage,
						note 	: notes[iNote]
					})
				};				
			};
			el.append(elPage);			
		}
		
		return null;	
	};

	/**
	 * Reders the Page Feedback existing notes
	 * @private	_renderPageFeedback
	 * @return	null
	 */
	var _renderPageFeedback = function(){
		
		// clear out any existing elements
		var el=$('#feedbackPanel .dki-feedback-page .content');
		$(el).children().each(function(i){
			$(this).remove()
		});
		
		// create the base element to insert into
		var elPage=document.createElement('div');
		elPage.className = 'page';
			
		// if no notes then display no feedback message
		if(_oActivePage.notes.length==0){
			var line=elPage.appendChild(document.createElement('p'))
			line.innerHTML = strings.txtTitleNoFeedBack;
			$(line).css('margin-bottom','20px')
		}else{
			
			for( var iNote=0; iNote < _oActivePage.notes.length; iNote++){
				_insertFeedbackNote({
					el 		: elPage,
					note 	: _oActivePage.notes[iNote]
				})
			};				
		};
			
		el.append(elPage);
		return null;
	};
	
	/**
	 * Inserts the standard dom structure for a feedback note on either the page feedback or all feedback tabs
	 * @private	_insertFeedbackNote
	 * @param	oConfig.el 		element to append to
	 * @paranm	oConfig.note 	note object to append;
	 * return	null
	 */
	var _insertFeedbackNote = function(oConfig){
		
		var method =oConfig.method==undefined?'append':oConfig.method;
	
		var wrapper = document.createElement('span');
	
		var dtm = new Date(oConfig.note.dateFiled);
		var line1 = (dtm.getMonth()+1) + '/' + dtm.getDate() + '/' + dtm.getFullYear();
		line1 += '&nbsp<i>'+oConfig.note.enteredBy.first + ' ' + oConfig.note.enteredBy.last+ '</i> ' + strings.txtLine1Noted
		wrapper.appendChild(document.createElement('p')).innerHTML=line1;

		var line2=wrapper.appendChild(document.createElement('p'))
		line2.innerHTML=oConfig.note.description.toString().replace(/\n/gi, "<br />");	
		$(line2).css('margin-bottom','20px');
		
		$(oConfig.el)[method](wrapper)
		
		return null;	
	};
	
	/**
	 * Initiates the layout of the feedback interface on FIRST CALL only and toggles its display
	 * @private	doLayout
	 * @return	true
	 */
	var _doLayout = function(){
		if(!_layoutComplete){
			
			_createLayoutStructure();
			
			_renderAllFeedback();
			
			_bindEvents();
			_layoutComplete=true;
			
			// by default initialize the page tab
			$('#feedbackPanel .menuHeader .menuTabs .selected').click() 
		}
		return true;
	}; //-> end doLayout
		
	/**
	 * Initializes the Feedback panel
	 * @method	init
	 * @param   player The course player object.
	 * @param	elementID	string id of element to render panel content to
	 * @param	dataStore	courseStore object
	 * @return	truee
	 */
	var initialize = function(player, elementID,dataStorage,astrings){
		presentationPlayer = player;
		strings = astrings;
		
		if(elementID==null || dataStorage==null || elementID==undefined || dataStorage==undefined){
			throw 'Missing required parameters';
			return false;
		};
				
		// set the global scopped variables 
		_container = $("#" + elementID);
		
		// where datastore is a DKI.courseStore
		_store = dataStorage;	
		
		// render the dom
		_doLayout()
		
		//$(_store).on(_store.ready, _doLayout)
		return true;	
	}; //-> end Initialize	

	/**
	 * Shows the Feedback panel
	 * @method	show
	 * @return	hidden state
	 */	
	var show = function(){
		_doLayout();
		_container.removeClass('hidden');
		_highlightFeedbackPage();
		_hidden=!_hidden;
		return _hidden		
	};
	
	/**
	 * Hides the Feedback panel
	 * @method	hide
	 * @return	hidden state
	 */
	var hide = function() {
		_container.addClass('hidden');
		_hidden=!_hidden;
		return _hidden;				
	};
	
	/**
	 * Toggles the view state of the feedback panel
	 * @method	toggleDisplay
	 * @return	hiden state
	 */
	var toggleDisplay = function(){
		return _hidden ? show():hide();
	};
	
	var menuNavigate = function(){
		_checkState({type:'pageCompleted'})
	}
	
	/**
	 * Exposed Feedback Methods
	 */
	return {
		init			: initialize,
		show 			: show,
		hide 			: hide,
		toggleDisplay	: toggleDisplay,
		menuNavigate	: menuNavigate
	};
	
}(); //-> end FeedBackPanel Singleton

