/**
 * Singleton representation of the glossary panel
 * @created	October 10, 2012
 * @author	Adam Reynolds
 * @version	1.0
 */
DKI.glossary = function(){
	var popupMarkup = '<div id="glossaryPopup" class="glossaryWindow loading">' +
			'<div class="glossaryPopupHeader">' +
				'<p id="glossaryPopupTerm"></p>' +
				'<div id="glossaryPopupPronunciation" class="jp-basic">' +					
					"<div class=\"jp-type-single\">" +
						"<div id=\"glossaryPopupPronunciation_player\" class=\"jp-audio\"></div>" +
						"<div class=\"jp-gui jp-basic jp-interface\">" +
							"<div class=\"jp-controls-holder\">" +
								"<ul class=\"jp-controls\">" +
									"<li><a class=\"jp-play\" tabindex=\"0\" role=\"button\">play</a></li>" +
									"<li><a class=\"jp-pause\" tabindex=\"0\" role=\"button\">pause</a></li>" +
								"</ul>" +
							"</div>" +
						"</div>" +
					"</div>" +
				'</div>' +
				'<div id="glossaryPopupClose" class="glossaryCloseButton"></div>' +
			'</div><div class="sectionBreak"></div>' +
			'<div class="glossaryPopupBody">' +
				'<div id="glossaryPopupDefinitionContainer">' +
					'<div id="glossaryPopupDefinition"></div>' +
				'</div>' +
			'</div> <div class="sectionBreak"></div>' +
			'<div id="linkContainer">' +
				'<a id="viewBrowserLink" title="View this term in the browser"></a>' +
			'</div>'+
		'</div>' +
		'<div id="glossaryHoverDefinition" class="glossaryWindow"></div>';
	var popupEl = {};
	var pronunciationPlayer = null;
	var terms = [];
	var settings = {};
	var initialize = function(config){
		terms = config.terms;
		settings = config.settings;
		language = config.languageString;
		$("#contentFrame").append(popupMarkup);
		popupEl.wrapper = $("#glossaryPopup");
		popupEl.term = $("#glossaryPopupTerm");
		popupEl.definition = $("#glossaryPopupDefinition");
		popupEl.player = $("#glossaryPopupPronunciation");
		popupEl.close = $("#glossaryPopupClose");
		popupEl.hoverDefinition = $("#glossaryHoverDefinition");	
		popupEl.viewBrowserLink = $('#viewBrowserLink');
		$("#glossaryPopup #glossaryPopupPronunciation_player").jPlayer({
			swfPath: settings.swfPath,
			cssSelectorAncestor: "#glossaryPopupPronunciation",
			ready: function(){
				pronunciationPlayer = $(this);
				popupEl.wrapper.hide();
				popupEl.wrapper.css("visibility", "visible");
				popupEl.wrapper.removeClass("loading");
				// Player Events
				contentApi.playerEvent.on('pageLoaded',closeAll);
				if(typeof player !== 'undefined') {
					player.navigation.forwardButton.on(settings.clickEvent,closeAll);	
				}		
			},
			supplied: "mp3",
			autohide: false,
			loop: false,
			preload: "auto",
			solution: "flash,html",
			size: {
				width: 0 + "px",
				height: 0 + "px"
			},
			muted: false,
			volume: 1
		});	
		popupEl.close.on(settings.clickEvent, function(){
			popupEl.wrapper.hide();
		});
		popupEl.wrapper.draggable({handle : $("#glossaryPopup .glossaryPopupHeader"), containment : ".dkiContentFrame.current"});
	};
	var calculateCoords = function(targetElement, display, offset) {
		if(DKI.courseStore) {
			var courseHeight = DKI.courseStore.course.get().height,
			courseWidth = DKI.courseStore.course.get().width
		}
		else {
			var courseHeight = DKI.PreviewPlayer.contentFrame.height(),
			courseWidth =DKI.PreviewPlayer.contentFrame.width()
		}
		var popupHeight = $(display).outerHeight(),
			popupWidth = $(display).outerWidth(),
			bodyLeftOffset = ($("html").width() - $("body").width()) /2,
			coordinates = {"x" : $(targetElement).offset().left - bodyLeftOffset, "y" : ($(targetElement).offset().top + targetElement.offsetHeight + offset) - player.barHeight};
		if(coordinates.x + popupWidth > courseWidth) {
			coordinates.x = courseWidth - (popupWidth + 20);
		}
		if(coordinates.y + popupHeight > courseHeight) {
			coordinates.y = coordinates.y - popupHeight - $(targetElement).outerHeight() - (offset);
		}
		return coordinates;
	}
	var showPopup = function(e){
		if(popupEl.wrapper.is(":visible") && settings.selectedTerm && settings.selectedTerm.glossaryid == $(this).data("id")) {
			popupEl.wrapper.hide();
		}
		else {
			settings.selectedTerm = getTerm($(this).data("id"));
			if(settings.selectedTerm) {
				hideDefinition();
				popupEl.term.text(settings.selectedTerm.term);
				popupEl.definition.html(settings.selectedTerm.definition);
				popupEl.viewBrowserLink.attr("href", "javascript:DKI.glossaryBrowse.selectTerm(" + '"' + settings.selectedTerm.id + '"' +")");
				popupCoords = calculateCoords(e.currentTarget, popupEl.wrapper, 5);
				popupEl.wrapper.css({
					"top": popupCoords.y + "px",
					"left": popupCoords.x + "px"
				});
				popupEl.wrapper.show();
				if(settings.selectedTerm.pronunciation_asset_filename){
					popupEl.player.show();
					pronunciationPlayer.jPlayer("setMedia", {
						"mp3": settings.assetURL + settings.selectedTerm.pronunciation_asset_filename
					});
				}
				else{
					popupEl.player.hide();
				}

				//Since IE6 does not support max-width/height, we use jquery to cover our bases here
				var defCont = $("#glossaryPopupDefinitionContainer");
				if(defCont.height() > 150) {
					defCont.height(150);
				}
			}
		}
	};
	
	var showDefinition = function(e, element) {
		if(!popupEl.wrapper.is(":visible")) {
			var glossaryTerm = getTerm($(element).data("id"));
			if(glossaryTerm) {
				var condensed = DKI.stripHTML(glossaryTerm.definition);
				condensed = condensed.length > 50 ? condensed.substr(0,47) + "..." : condensed;
				popupEl.hoverDefinition.html("<p style='display:inline;'>" + condensed + "</p>");
				var popupCoords = calculateCoords(e.currentTarget, popupEl.hoverDefinition, 5);
				popupEl.hoverDefinition.css({
					"top" : popupCoords.y + "px",
					"left": popupCoords.x + "px"
				});
				popupEl.hoverDefinition.show();
			}
		}
	}

	var hideDefinition = function(e) {
		popupEl.hoverDefinition.hide();
	}

	var closeAll = function() {
		popupEl.hoverDefinition.hide();
		popupEl.wrapper.hide();
	}

	var getTerm = function(id){	
		for(var i = 0; i < terms.length; i++){			
			if(terms[i].glossaryid == id){
				return terms[i];
			}
		}
	};


	return {
		init: initialize,
		showPopup: showPopup,
		showDefinition : showDefinition,
		hideDefinition : hideDefinition
	};
	
}();

