var DKI;
if (!DKI) {DKI = {};}
DKI.PlayerNavigation = function (player) {
	this.exitButton = $("#exitButton");	
	this.audioButton = $("#audioButton");
	this.replayButton = $("#replayButton");
	this.backButton = $("#backButton");
	this.forwardButton = $("#forwardButton");
	this.menuButton = $("#menuButton");
	this.listButton = $("#listButton");
	this.moreButtons = $("#moreButtons");
	this.moreOverlay = $("#moreOverlay");
	this.transcriptButton = $("#transcriptButton");
	this.glossaryButton = $("#glossaryButton");
	this.showForward = true;
	this.showBack = true;
	this.showAudio = true;
	this.showReload = true; 
	this.showMenu = true;
	this.audioState = "on";
	this.menuState = false;
	this.moreState = false;
	this.glossaryState = false;
	this.clickEvent = player.settings.clickEvent;
	this.initEvents();
};

DKI.PlayerNavigation.prototype.initEvents = function () {
	var that = this;

	$(document.body).delegate(".navButton", "keypress", function (e) {
		if (e.which === 13 || e.which === 32) {
			$(this).click();
			return false;
		}
	});

	//Setup DOM event handlers	
	$(document.body).on(this.clickEvent, "#exitButton", function (e) {
		e.stopPropagation();
		$(that).trigger(DKI.PlayerNavigation.events.exit);
		return false;
	});

	$(document.body).on(this.clickEvent, "#audioButton", function (e) {
		e.stopPropagation();
		if (!$(this).hasClass("disabled")) {
			if (that.audioState === "on") {
				that.audioState = "off";
			}
			else {
				that.audioState = "on";
			}
			that.audioButton.toggleClass("muted");
			$(that).trigger(DKI.PlayerNavigation.events.audioToggle, [that.audioState]);
			that.hideMore();
		}

		return false;
	});

	$(document.body).on(this.clickEvent, "#backButton", function (e) {
		e.stopPropagation();
		if (!$(this).hasClass("disabled")) {
			$(that).trigger(DKI.PlayerNavigation.events.back);
		}
		
		return false;
	});

	$(document.body).on(this.clickEvent, "#forwardButton", function (e) {
		e.stopPropagation();
		if (!$(this).hasClass("disabled")) {
			$(that).trigger(DKI.PlayerNavigation.events.forward);
		}
		return false;
	});

	$(document.body).on(this.clickEvent, "#menuButton", function (e) {		
		e.stopPropagation();
		if (!$(this).hasClass("disabled")) {
			//Since the menu button is hidden while menu is open, it can only be used to open the menu
			if (that.menuState) {
				that.menuState = false;
			}
			else {
				that.menuState = true;
			}
			$(that).trigger(DKI.PlayerNavigation.events.menuToggle, [that.menuState]);
			that.hideMore();
		}

		return false;

	});

	$(document.body).on(this.clickEvent, "#replayButton", function (e) {
		e.stopPropagation();
		if (!$(this).hasClass("disabled")) {
			$(that).trigger(DKI.PlayerNavigation.events.reload);
			that.hideMore();
		}		
		return false;
	});

	$(document.body).on(this.clickEvent, "#listButton", function (e) {
		e.stopPropagation();
		that.onListClick(e);
		return false;
	});

	$(document.body).on(this.clickEvent, "#transcriptButton", function (e) {
		e.stopPropagation();
		if (!$(this).hasClass("disabled")) {
			$(that).trigger(DKI.PlayerNavigation.events.transcriptToggle);
		}
		return false;
	});

	$(document.body).on(this.clickEvent, "#glossaryButton", function (e) {		
		e.stopPropagation();
		if (!$(this).hasClass("disabled")) {
			//Since the menu button is hidden while menu is open, it can only be used to open the menu
			if (that.glossaryState) {
				that.glossaryState = false;
			}
			else {
				that.glossaryState = true;
			}
			$(that).trigger(DKI.PlayerNavigation.events.glossaryToggle, [that.glossaryState]);
			that.hideMore();
		}

		return false;

	});

	if (this.moreOverlay) {
		$(document.body).on(this.clickEvent, "#moreOverlay", function (e) {
			e.stopPropagation();
			that.hideMore();
			that.moreOverlay.css("display", "none");
			return false;
		});
	}
};

/* Methods to fire navigation events */
DKI.PlayerNavigation.prototype.fireBack = function () {
	$(this).trigger(DKI.PlayerNavigation.events.back);
};

DKI.PlayerNavigation.prototype.fireForward = function () {
	$(this).trigger(DKI.PlayerNavigation.events.forward);
};
/* End fire methods */

/* show/hide navigation elements */
DKI.PlayerNavigation.prototype.hideTranscriptButton = function () {
	this.transcriptButton.css("display", "none");
};

DKI.PlayerNavigation.prototype.showTranscriptButton = function () {
	this.transcriptButton.css("display", "");
};

DKI.PlayerNavigation.prototype.disableTranscriptButton = function () {
	this.transcriptButton.addClass("disabled");
};

DKI.PlayerNavigation.prototype.enableTranscriptButton = function () {
	this.transcriptButton.removeClass("disabled");
};

DKI.PlayerNavigation.prototype.hideAudioButton = function () {
	this.audioButton.addClass("disabled");
};

DKI.PlayerNavigation.prototype.showAudioButton = function () {
	this.audioButton.removeClass("disabled");
};

DKI.PlayerNavigation.prototype.hideBackButton = function () {
	this.backButton.addClass("disabled");
};

DKI.PlayerNavigation.prototype.showBackButton = function () {
	this.backButton.removeClass("disabled");
};

DKI.PlayerNavigation.prototype.hideForwardButton = function () {
	this.forwardButton.addClass("disabled");
};

DKI.PlayerNavigation.prototype.showForwardButton = function () {
	this.forwardButton.removeClass("disabled");
};

DKI.PlayerNavigation.prototype.hideMenuButton = function () {
	this.menuButton.addClass("disabled");
};

DKI.PlayerNavigation.prototype.showMenuButton = function () {
	this.menuButton.removeClass("disabled");
};

DKI.PlayerNavigation.prototype.hideReloadButton = function () {
	this.replayButton.addClass("disabled");
};

DKI.PlayerNavigation.prototype.showReloadButton = function () {
	this.replayButton.removeClass("disabled");
};

DKI.PlayerNavigation.prototype.showScreenCountDisplay = function () {
	$("#screenCount").css("display", "");
};

DKI.PlayerNavigation.prototype.hideScreenCountDisplay = function () {
	$("#screenCount").css("display", "none");
};

DKI.PlayerNavigation.prototype.hideGlossaryButton = function () {
	this.glossaryButton.css("display", "none");
};

DKI.PlayerNavigation.prototype.showGlossaryButton = function () {
	this.glossaryButton.css("display", "");
};

DKI.PlayerNavigation.prototype.disableGlossaryButton = function () {
	this.glossaryButton.addClass("disabled");
};

DKI.PlayerNavigation.prototype.enableGlossaryButton = function () {
	this.glossaryButton.removeClass("disabled");
};

/* end show/hide methods */

DKI.PlayerNavigation.prototype.setTotalScreens = function (number) {
	$("#totalScreens").text(number);
};

DKI.PlayerNavigation.prototype.setCurrentScreen = function (number) {
	$("#currentScreen").text(number);
};

DKI.PlayerNavigation.prototype.setModuleName = function (name) {
	$("#moduleName").text(name);
};

DKI.PlayerNavigation.prototype.setObjectName = function (name) {
	$("#objectName").text(name);
};

DKI.PlayerNavigation.prototype.setMenuState = function (state) {
	this.menuState = state;
};

DKI.PlayerNavigation.prototype.setGlossaryState = function(state) {
	this.glossaryState = state;
};

DKI.PlayerNavigation.prototype.contentMode = function () {
	this.showAudioButton();
	this.showReloadButton();
	this.showMenuButton();
	this.showScreenCountDisplay();
};

DKI.PlayerNavigation.prototype.testMode = function (showNav) {
	if (!showNav) {
		this.hideForwardButton();
		this.hideBackButton();
	}
	this.showAudioButton();
	this.hideReloadButton();
	this.hideMenuButton();
	this.showScreenCountDisplay();
};

DKI.PlayerNavigation.prototype.endMode = function () {
	this.hideForwardButton();
	this.hideBackButton();
	this.hideReloadButton();
	this.hideAudioButton();
	this.hideMenuButton();
	this.hideScreenCountDisplay();
	this.setObjectName("");
	this.setModuleName("");
};

DKI.PlayerNavigation.prototype.onListClick = function () {
	this.menuState = false;		
	$(this).trigger(DKI.PlayerNavigation.events.menuClose);
 			
	if (this.moreState) {
		this.moreButtons.css("display", "none");	
		this.moreOverlay.css("display", "none");
		this.moreState = false;
	}
	else {
		this.moreButtons.css("display", "block");
		this.moreOverlay.css("display", "block");
		this.moreState = true;
	}
};

DKI.PlayerNavigation.prototype.hideMore = function () {
	if (this.moreState) {
		this.moreButtons.css("display", "none");
		this.moreOverlay.css("display", "none");
		this.moreState = false;
	}

}

DKI.PlayerNavigation.events = {
	audioToggle: "audioToggle",
	back: "back",
	forward: "forward",
	menuToggle: "menuToggle",
	menuClose: "menuClose",
	audioToggle: "audioToggle",
	reload: "reload",
	exit: "exit",
	transcriptToggle: "transcriptToggle",
	glossaryToggle: "glossaryToggle"
};


