DKI.SCORMAssessment = DKI.Assessment.extend({
	prototype: {
		init: function(module, prepost, test, scormAPI) {
			this._parent(module, prepost, test);
			this.scormAPI = scormAPI;
		},
		scoreQuestion: function (question, questionObject, score) {
			this._parent(question, questionObject, score);
			this.recordInteraction(question, questionObject, score);
		},
		recordInteraction: function (question, questionObject, score) {
			questionObject.questionOptions = question.options;
			questionObject.id = question.id;
			questionObject.score = score;
			questionObject.weight = question.weight;
			switch(questionObject.type) {
				case 1:
					this.recordTrueFalse(questionObject);
					break;
				case 2:
					this.recordMultipleChoice(questionObject);
					break;
				case 3:
					this.recordFreeForm(questionObject);
					break;
				case 4: 
					this.recordFillBlanks(questionObject);
					break;
				case 5:
					this.recordPulldown(questionObject);
					break;
				case 6:
					this.recordDragDrop(questionObject);
					break;
			}
		},
		recordTrueFalse: function (questionObject) {
			var actualChoice = questionObject.options[0] == 1 ? true : false;
			var correctChoice = questionObject.questionOptions[0].correct;
			var isCorrect = questionObject.score == 100 ? true : false;

			this.scormAPI.RecordTrueFalseInteraction(
				questionObject.id,
				actualChoice,
				isCorrect,
				correctChoice,
				questionObject.body,
				questionObject.weight,
				questionObject.latency);

		},
		recordMultipleChoice: function (questionObject) {
			var letters = ["a", "b", "c", "d", "e", "f", "g", "h"];
			var actualChoice = [];
			var correctChoice = [];
			var isCorrect = questionObject.score == 100 ? true : false;
			for (var i = 0; i < questionObject.options.length; i ++) {
				if (questionObject.options[i].checked == 1) {
					actualChoice[actualChoice.length] = this.scormAPI.CreateResponseIdentifier(letters[i], questionObject.options[i].text);
				}
				if (questionObject.questionOptions[i].correct) {
					correctChoice[correctChoice.length] = this.scormAPI.CreateResponseIdentifier(letters[i], questionObject.options[i].text);
				}
			}
			this.scormAPI.RecordMultipleChoiceInteraction(
				questionObject.id,
				actualChoice,
				isCorrect,
				correctChoice,
				questionObject.body,
				questionObject.weight,
				questionObject.latency);
		},
		recordFillBlanks: function (questionObject) {
			var actualChoice = "";
			var correctChoice = "";
			var currentCorrectChoice;
			var isCorrect = questionObject.score == 100 ? true : false;
			var matchedChoice;
			for (var i = 0; i < questionObject.options.length; i ++) {
				actualChoice += questionObject.options[i];
				currentCorrectChoice = questionObject.questionOptions[i].correctValues;
				matchedChoice = currentCorrectChoice[0];
				for (var j = 0; j < currentCorrectChoice.length; j ++) {
					if ($.trim(questionObject.options[i].toString().toLowerCase()) == $.trim(currentCorrectChoice[j].toString().toLowerCase())) {
						matchedChoice = currentCorrectChoice[j];
						break;
					}
				}
				correctChoice += matchedChoice;
				if (i < questionObject.options.length - 1) {
					actualChoice += ",";
					correctChoice += ",";
				}
			}
			this.scormAPI.RecordFillInInteraction(
				questionObject.id,
				actualChoice,
				isCorrect,
				correctChoice,
				questionObject.body,
				questionObject.weight,
				questionObject.latency);
		},
		recordPulldown: function (questionObject) {
			var actualChoice = [];
			var correctChoice = [];
			var optionChoices = [];
			var isCorrect = questionObject.score == 100 ? true : false;
			for (var i = 0; i < questionObject.options.length; i ++) {
				optionChoices = [];
				optionChoices[0] = questionObject.questionOptions[i].correctValue;
				optionChoices = optionChoices.concat(questionObject.questionOptions[i].incorrectValues);				
				correctChoice[correctChoice.length] = this.scormAPI.CreateResponseIdentifier("1", optionChoices[0]);
				for (var j = 0; j < optionChoices.length; j ++) {
					if (questionObject.options[i].toString().toLowerCase() == optionChoices[j].toString().toLowerCase()) {
						actualChoice[actualChoice.length] = this.scormAPI.CreateResponseIdentifier((j + 1).toString(), questionObject.options[i].toString().toLowerCase());
					}
				}
			}
			this.scormAPI.RecordSequencingInteraction(
				questionObject.id,
				actualChoice,
				isCorrect,
				correctChoice,
				questionObject.body,
				questionObject.weight,
				questionObject.latency);

		},
		recordFreeForm: function (questionObject) {
			this.scormAPI.RecordFillInInteraction(
				questionObject.id,
				questionObject.options[0],
				this.scormAPI.INTERACTION_RESULT_NEUTRAL,
				questionObject.options[0],
				questionObject.body,
				questionObject.weight,
				questionObject.latency);

		},
		recordDragDrop: function (questionObject) {
			var actualChoices = [];
			var correctChoices = [];
			var optionChoices = [];
			var source = null;
			var target = null;
			var isCorrect = questionObject.score == 100 ? true : false;
			for (var i = 0; i < questionObject.options.length; i ++) {
				source = this.scormAPI.CreateResponseIdentifier((i + 1).toString(), questionObject.options[i].title);
				target = this.scormAPI.CreateResponseIdentifier((i + 1).toString(), questionObject.options[i].target.title);
				actualChoices[actualChoices.length] = new this.scormAPI.MatchingResponse(source, target);
				var found = false;
				var questionOption = null;
				for (var j = 0; j < questionObject.questionOptions.length; j++) {
					if(questionObject.questionOptions[j].element_id == questionObject.options[i].elementId){
						questionOption = questionObject.questionOptions[j];
					}
				}
				optionChoices = [];				
				optionChoices = optionChoices.concat(questionOption.correctTargets);
				if (optionChoices.length > 0) {
					for (var j = 0; j < optionChoices.length; j++) {												
						if (questionObject.options[i].target.id == optionChoices[j]) {																
							correctChoices[correctChoices.length] = new this.scormAPI.MatchingResponse(source, target);
							found = true;
						}
					}
					if(!found){
						//the target it was dropped in is not in the correctTargets array...
						var defaultCorrectTarget = this.scormAPI.CreateResponseIdentifier((i + 1).toString(), optionChoices[0]);
						correctChoices[correctChoices.length] = new this.scormAPI.MatchingResponse(source, defaultCorrectTarget);
					}
				}
				else if (!questionObject.options[i].target.id){
					//if they didnt drop it anywhere and weren't supposed to, they're right.
					correctChoices[correctChoices.length] = new this.scormAPI.MatchingResponse(source, target);					
				}
				else{
					//if they werent supposed to select and they did
					var nullTarget = this.scormAPI.CreateResponseIdentifier((i + 1).toString(), "No Target");
					correctChoices[correctChoices.length] = new this.scormAPI.MatchingResponse(source, nullTarget);					
				}
			}
			this.scormAPI.RecordMatchingInteraction(
				questionObject.id,
				actualChoices,
				isCorrect,
				correctChoices,
				questionObject.body,
				questionObject.weight,
				questionObject.latency);

		}
	}
});
