DKI.SCORMDataStorage = DKI.DataStorage.extend({
	init: function (courseStructure, behaviour, aStrings, scormAPI) {
		var that = this;
		this._parent(courseStructure, behaviour, aStrings);
		this.scormAPI = scormAPI;
		
	},
	createAssessment: function(module, prePost, test) {
		return new DKI.SCORMAssessment(module, prePost, test, this.scormAPI);
	},
	initEvents: function () {
		var that = this;
		$(this).on(DKI.DataStorage.events.pageSelected, function (e, page) {
			that.saveCompletion(page);		
		});
		this._parent.initEvents();
	},
	loadCompletion: function (autoResume) {
		this.bookmark = this.scormAPI.GetBookmark();
		if (this.bookmark === "") {
			this.bookmark = null;
		}
		var passMark = this.scormAPI.GetPassingScore();
		var entryMode = this.scormAPI.GetEntryMode();
		var courseStatus = this.scormAPI.GetStatus();
		var chunkData = this.scormAPI.GetDataChunk();
		var that = this;
		var completionObject;
		if (passMark > 0) {
			this.passMark = passMark;
		}
		if(courseStatus === this.scormAPI.LESSON_STATUS_PASSED || courseStatus === this.scormAPI.LESSON_STATUS_FAILED || courseStatus === this.scormAPI.LESSON_STATUS_COMPLETED) {
			completionObject = JSON.parse(chunkData);
			this.markCourseComplete();
			//Loop through tests
			for (var i = 0; i < completionObject.t.length; i ++) {
				if (completionObject.t[i].pre) {
					this.setModulePre(completionObject.t[i].id, completionObject.t[i].pre);
				}
				if (completionObject.t[i].post) {
					this.setModulePost(completionObject.t[i].id, completionObject.t[i].post);
				}
			}
		}
		else if (chunkData !== ""){
			completionObject = JSON.parse(chunkData);	
				
			for (var i = 0; i < completionObject.m.length; i ++) {
				this.markModuleComplete(completionObject.m[i]);	
			}
			for (var i = 0; i < completionObject.o.length; i ++) {
				this.markObjectComplete(completionObject.o[i]);
			}
			for (var i = 0; i < completionObject.p.length; i ++) {
				this.markPageComplete(completionObject.p[i]);
			}
			for (var i = 0; i < completionObject.t.length; i ++) {
				if (completionObject.t[i].pre) {
					this.setModulePre(completionObject.t[i].id, completionObject.t[i].pre);
				}
				if (completionObject.t[i].post) {
					this.setModulePost(completionObject.t[i].id, completionObject.t[i].post);
				}
			}
		}
		this._parent.loadCompletion(autoResume);	
	},
	saveCompletion: function (page) {
		if (page) {
			this.scormAPI.SetBookmark(page.id, page.title);
		}
		this.scormAPI.SetProgressMeasure(this.getCourseCompletion() / 100);	
		this.scormAPI.SetDataChunk(this.serializeCompletion());
		this.scormAPI.CommitData();
	},
	serializeCompletion: function () {
		var currentMod, currentObj, testRecord;
		var completionObj = {
			m: [],
			o: [],
			p: [],
			t: []
		};
		for (var i = 0; i < this.courseStructure.modules.length; i ++) {
			currentMod = this.courseStructure.modules[i];
			if (currentMod.complete) {
				completionObj.m[completionObj.m.length] = currentMod.id;
			}
			else {
				for (var j = 0; j < currentMod.objects.length; j ++) {
					currentObj = currentMod.objects[j];
					if (currentObj.complete) {
						completionObj.o[completionObj.o.length] = currentObj.id;
					}
					else {
						for (var k = 0; k < currentObj.pages.length; k ++) {
							if (currentObj.pages[k].complete) {
								completionObj.p[completionObj.p.length] = currentObj.pages[k].id;
							}
						}
					}
				}
			}
			if (currentMod.pre || currentMod.post) {
				testRecord = {
					id: currentMod.id
				};
				if (currentMod.pre) {
					testRecord.pre = currentMod.pre;
				}
				if (currentMod.post) {
					testRecord.post = currentMod.post;
				}
				completionObj.t[completionObj.t.length] = testRecord;
			}
		}
		return JSON.stringify(completionObj);
	},
	setModulePre: function (moduleID, preScore) {
		var module = this.findModule(moduleID);
		module.pre = preScore;
	},
	setModulePost: function (moduleID, postRecord) {
		var module = this.findModule(moduleID);
		module.post = postRecord;
	},
	markCourseComplete: function () {
		for (var i = 0; i < this.courseStructure.modules.length; i ++) {
			this.markModuleComplete(this.courseStructure.modules[i]);	
		}
	},
	//Event handler functions
	onCourseComplete: function () {
		this.scormAPI.SetProgressMeasure(1.0);
		this._parent.onCourseComplete();
		switch (this.lessonStatus) {
			case "passed":
				this.scormAPI.SetScore(this.getCourseScore(), 100, 0);
				this.scormAPI.SetPassed();
				break;
			case "failed":
				this.scormAPI.SetScore(this.getCourseScore(), 100, 0);
				this.scormAPI.SetFailed();
				break;
			case "complete":
				this.scormAPI.objLMS.SetCompleted();
				this.scormAPI.SetPassed();
				break;
		}
		this.scormAPI.CommitData();
	},
	onAssessmentComplete: function (event) {
		var nextModule;
		this._parent.onAssessmentComplete();
		if (this.scormAPI.objLMS.Standard === "SCORM2004") { 
			this.scormAPI.SetScore(this.getCourseScore(), 100, 0);
		}
		if (this.courseStructure.testOnly == "1") {
			nextModule = this.getNextModule(this.assessment.module);
			if (nextModule) {
				this.scormAPI.SetBookmark(nextModule.id, nextModule.name);
			}
			else {
				this.scormAPI.SetBookmark("", "");
			}
		}
		this.saveCompletion();
	}
});
