/*
 * This file provides for the tree in the left hand panel of the player
 * Given that this tree is written in jquery and is ONLY ever created once it is written as a singletong
 * using the Crockford module patter. This is done in order to reduce the memory footprint of the tree
 * If need be this can easily be converted into a proper instantiable class with a few minor changes
 * 
 * The tree is initialized via the init method which is passed the elementID and dataStorage oject
 * 
 * @version		1.0
 * @created		August 17, 2001
 * @author		Keith Chadwick
 * @namespace	DKI.PlayerTree
 */

var DKI;
if (!DKI) {
	DKI = {};
}

DKI.PlayerTree= function(){
	
	/*
	 * Private declarations
	 */
	var _elementID = null;		// passed element id on init
	var _container = null;		// root container of tree
	var _isVisible = false;		// indicates if the menu is displayed
	
	var store = null;			// DKI.courseStore pointer rather than full path
	var presentationPlayer;
	
	/*
	 * Renders the course data based on the passed dataStorage
	 * @private _renderCourseTree 
	 * @return null
	 */
	function _renderCourseTree (){

		// TODO: Destroy the dom properly, although is should ALWAYS be empty
		_container.html("");
		
		var forceSequential=playerBehaviour.forceSequential;
		
		if (store !== undefined && store !== null) {
			
			// Create the root node of the tree
			var course = store.course.get();
			
			var cEl = document.createElement("div");
			cEl.id = course.key
			cEl.className = 'treeNode collapse ' + store.NodeType.course;

			var child=cEl.appendChild(document.createElement("a"));
			child.innerHTML = "<span title='" + course.name + "'>" + (course.name.length > 45 ? course.name.substr(0, 42) + "..." : course.name) + "</span>";
			if (course.complete) {
				child.className = "finished";
			}
			else if (course.started) {
				child.className = "started";
			}
			else {
				child.className = "unstarted";
			}
			child.className += " collapse";
			child.id = course.key + 'Link';
			
			// enumerate the modules
			for(var mI in course.childKeys){
				
				// Create the module node
				var module = store.module.get(course.childKeys[mI]);
				
				var mEl = document.createElement("div");		
				mEl.id = module.key;
				mEl.className = 'treeNode collapse ' + store.NodeType.module;				
				
				// create the child node
				var child=mEl.appendChild(document.createElement("a"));
				child.innerHTML = "<span title='" + module.name + "'>" + (module.name.length > 35 ? module.name.substr(0, 32) + "..." : module.name) + "</span>";
				if (module.complete) {
					child.className = 'finished';
				}
				else if (module.started) {
					child.className = 'started';
				}
				else {
					child.className = 'unstarted';
				}
				child.className += ' collapse';
				child.id = module.key + "link";
				
				cEl.appendChild(mEl)
				
				// enumerate the objects
				for(var oI in module.childKeys){

					var object = store.get(module.childKeys[oI]);
										
					// create the object node
					var oEl = document.createElement("div");
					oEl.className = 'treeNode collapse ' + store.NodeType.object;	
					oEl.id = object.key;
					
					// create the child anchor node
					var child=oEl.appendChild(document.createElement("a"));
					child.innerHTML = "<span title='" + object.name + "'>" + (object.name.length > 35 ? object.name.substr(0, 32) + "..." : object.name) + "</span>";
					if (object.complete) {
						child.className = 'finished';
					}
					else if (object.started) {
						child.className = 'started';
					}
					else {
						child.className = 'unstarted';
					}
					child.className += ' collapse';
					child.id = object.key + "link";
					
					mEl.appendChild(oEl);	
					
					for(var pI in object.childKeys){
						
						var page = store.get(object.childKeys[pI])
						if(page.nodeType==store.NodeType.page){
						
							var pEl = document.createElement("div");
							pEl.id = page.key;
							pEl.className = 'treeNode ' + store.NodeType.page + (forceSequential && !page.complete?' disabled':'');
	
							var child = pEl.appendChild(document.createElement("a"));
							child.innerHTML = "<span title='" + page.title + "'>" + (page.title.length > 35 ? page.title.substr(0, 32) + "..." : page.title) + "</span>";
							if (page.complete) {
								child.className = 'finished';
							}
							else {
								child.className = 'unstarted';
							}
							child.id = page.key + "link";
							
							oEl.appendChild(pEl);						
						}
					};	// end page loop			
				}; // end object loop
			}; // end module loop
			_container.append(cEl);
		}
		else {
			_container.html("No outline available");
		};
		return null;
	}; //-> end _renderCourseTree
				
	/*
	 * Binds the click methods to the tree
	 * @private	_initEvents
	 * @return null
	 */
	function _initEvents(){
		$(_container).delegate(".module,.object,.course,.page", "click", function (e) {
			_onNodeClick(this);
			return false;
		});
		return null;
	}; //-> end _initEvents

	/*
	 * Processes the passed el dom element and collapses or expands the child elements with a css class
	 * @private	_onNodeClick
	 * @param	el	dom object pointer
	 * @return	true
	 */
	var _onNodeClick = function(el){
		
		var oEl = $(el);
		var node = store.get(el.id)

		
		if(node.nodeType==store.NodeType.page){
			
			if(oEl.hasClass('disabled')){return}
			
			SelectPage(node.id)
			DKI.PlayerTree.pageClicked(DKI.PlayerTree.pageClicked, node.id);	
			DKI.FeedbackPanel.menuNavigate()
		}else{
			oEl.toggleClass('collapse')
			$(oEl.children()[0]).toggleClass('collapse')
		};
				
		return true;
	}; //-> end _onNodeClick
	
	/*
	 * Public Methods
	 */
	
	/*
	 * Initializes the tree at passed dom element based on based course store
	 * @method	init
	 * @param   player The course player object
	 * @param	elementID	Required string ID of existing dom element 
	 * @param	dataStorage	Required object representing the course store
	 * @return	null
	 */
	var InitializeTree = function(player, elementID, dataStorage){
		
		presentationPlayer = player;
		//strings = player.strings;
		if(elementID==null || dataStorage==null || elementID==undefined || dataStorage==undefined){
			//throw {'Missing required parameters'};
			return
		};
		
		// set the global scopped variables 
		_container = $("#" + elementID);
		_container.addClass("dki-PlayerTree");
		
		// where datastore is a DKI.courseStore
		store = dataStorage;
		
		// called the private methods 
		_renderCourseTree();
		_initEvents();
		return null
	}; //-> end InitalizeTree

	/*
	 * Sets the complete status icon based on the passed object type and id
	 * @method	setComplete
	 * @param	config.type	string object type see exposed nodeType
	 * @param	config.id	id of object
	 * @return	null
	 */
	var SetComplete = function(config){
		
		// we only EVER need to deal with nodes of type page
		if(config.type!=store.NodeType.page){return}

		// get the element of the passed type id  and set the status to complete
		var node=$('#'+config.type+config.id+'link');
		node.removeClass("unstarted");
		node.removeClass("started");
		node.addClass("finished");
		
		// increment the parent counter returns true if the parent is now complete
		// start point will be object
		var page = store.page.get(config.type+config.id)
		if(!page.complete){
			page.complete=true;
			store.page.set(page);
			store.setParentCount(page.key)
			setUIState(
				store.module.get(
					setUIState(store.object.get(page.parentId))	// object
				)
			)
			setUIState(store.course.get())
		}
		return
 
		/*
		 * applies the appropriate styles to the node
		 * @private setUIState
		 * @param	store node
		 * @return 	parentId as string 
		 */
		function setUIState(node){
			var el=$($('#'+node.key).children()[0]);
			el.removeClass('unstarted').removeClass('finished').removeClass('started')
			el.addClass(node.complete ? 'finished':'started')
			return node.parentId
		};
	}; //-> end SetComplete
	
	
	/*
	 * Selects the passed page id ensures any other page id is not selected and expands parent nodes
	 * @method	selectPage
	 * @param	id	integer page id
	 * @return 	null
	 */
	var SelectPage = function(id){

		var selectedNode = $("#" + store.NodeType.page + id + "link");
		var outline = $("#outlineContent");
		var root = $(".treeNode.course", outline);
		//var parentObject = selectedNode.attr("data-parentid");
		
		// make sure no other page node is selected
		$("a.selected", _container).each(function (i) {
			$(this).removeClass("selected");
		});
		
		// select the passed page
		selectedNode.addClass("selected");
		selectedNode.parent().removeClass("disabled");
		
		// ensure we scroll to the node so that it is visible
		scrollToSelected();
		
		// we also need to bold the parent object node so remove the active class then apply to the current
		$("div.object a", _container).each(function (i) {
			$(this).removeClass("active");
		});
		$($('#'+store.NodeType.page+id).parent().children()[0]).addClass('active')
		
		// move up the dom tree until no more parents and ensure they are
		// expanded		
		var pNode=selectedNode.parent()
		while(pNode!=null){
			if(pNode.hasClass('treeNode')){
				pNode.removeClass('collapse')
				$(pNode.children()[0]).removeClass('collapse')
				pNode=pNode.parent()
			}else{
				pNode=null
			}
		};
		return null;
	}; //-> end SelectPage

	var scrollToSelected = function () {
		var outline = $(document.getElementById("outlineContent"));
		var selectedNode = $("a.selected", outline);
		var root = $(".treeNode.course", outline);

		//We can't use the position() method, as it changes based on scroll position.
		//So we calculate the top of the outline and the selected node based on the document, and
		//scroll by the difference
		outline.scrollTop(selectedNode.offset().top - root.offset().top);
	};
	
		
	return {
		init				: InitializeTree,
		scrollToSelected	: scrollToSelected,
		setComplete			: SetComplete,
		selectPage			: SelectPage,
		setCourse			: InitializeTree,
		
		
		// empty event binding for menu
		pageClicked			: null		
	};
		
}(); //-> end PlayerTree
